package com.ruoyi.ss.callback.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.constant.Constants;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.pay.wx.domain.NotifyEventType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.http.HttpUtils;
import com.ruoyi.ss.callback.domain.CallbackLog;
import com.ruoyi.ss.callback.mapper.CallbackLogMapper;
import com.ruoyi.ss.callback.service.ICallbackService;
import com.ruoyi.ss.changeBalance.service.IChangeBalanceService;
import com.ruoyi.ss.channel.service.IChannelService;
import com.ruoyi.ss.channel.domain.ChannelVO;
import com.ruoyi.ss.order.domain.Order;
import com.ruoyi.ss.order.domain.OrderVO;
import com.ruoyi.ss.orderOper.service.IOrderOperService;
import com.ruoyi.ss.payBill.domain.PayBillVO;
import com.ruoyi.ss.payBill.domain.enums.PayBillStatus;
import com.ruoyi.ss.payBill.service.PayBillService;
import com.ruoyi.ss.refund.domain.Refund;
import com.ruoyi.ss.order.service.IOrderService;
import com.ruoyi.ss.refund.service.IRefundService;
import com.ruoyi.ss.user.service.IUserService;
import com.wechat.pay.java.core.RSAAutoCertificateConfig;
import com.wechat.pay.java.core.notification.Notification;
import com.wechat.pay.java.core.notification.NotificationConfig;
import com.wechat.pay.java.core.notification.NotificationParser;
import com.wechat.pay.java.core.notification.RequestParam;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.refund.model.RefundNotification;
import com.wechat.pay.java.service.refund.model.Status;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;


/**
 * 订单支付回调 CallbackService 实现类
 */
@Service
public class CallbackServiceImpl implements ICallbackService {

    private static final Logger logger = LoggerFactory.getLogger(CallbackServiceImpl.class);

    @Resource
    private IOrderService orderService;

    @Resource
    private IRefundService etRefundService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Resource
    private CallbackLogMapper callbackLogMapper;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private IChannelService rlChannelService;

    @Autowired
    private IUserService userService;

    @Autowired
    private IChangeBalanceService changeBalanceService;

//    @Autowired
//    private IRlDeliveryOrderService deliveryOrderService;
//
//    @Resource
//    private IRlDividendDetailService dividendDetailService;

//    @Resource
//    private IRlUserExtService userExtService;
//
//    @Resource
//    private IRlAgentService agentService;

    @Autowired
    private IOrderOperService orderOperService;

    @Autowired
    private PayBillService paymentService;


    /**
     * 微信支付回调
     */
    @Override
    @Transactional
    public void weChat(HttpServletRequest request) {
        String body = HttpUtils.getBody(request);
        logger.info("【微信支付回调】接收对象（未验签） : " + JSON.toJSONString(body));
        CallbackLog etCallbackLog = new CallbackLog();
        etCallbackLog.setBody2(body);
        etCallbackLog.setType("1");
        // 解析通知数据
        Notification notification = JSON.parseObject(body, Notification.class);

        //异步保存回调日志
        asynchronousSaveCallbackLog(etCallbackLog);

        // 支付成功通知
        if (NotifyEventType.TRANSACTION_SUCCESS.getValue().equals(notification.getEventType())) {
            // 验签、解密并转换成 Transaction
            Transaction transaction = checkAndParse(request, body, Transaction.class);
            if (Transaction.TradeStateEnum.SUCCESS.equals(transaction.getTradeState())) {
                // 充值成功后的业务处理
                etCallbackLog.setBody(JSON.toJSONString(transaction));
                businessHandle(transaction.getOutTradeNo(), ServiceConstants.PAY_TYPE_WX);//业务处理
            }
        }
    }

    /**
     * 支付成功后的业务处理
     *  1. 更改订单状态，订单到期时间修改，账变记录保存至订单
     *  2. 生成配送工单
     *  3. 生成账变记录
     */
    @Override
    public void businessHandle(String outTradeNo,String payType) {
        PayBillVO payBillVO = paymentService.selectByPayNo(outTradeNo);

        ServiceUtil.assertion(!PayBillStatus.PAYING.getStatus().equals(payBillVO.getStatus()), "该支付订单不是正在支付的支付订单");

        OrderVO orderVO = orderService.selectRlOrderByOrderNo(payBillVO.getOrderNo());
        ServiceUtil.assertion(orderVO == null, "订单不存在");

        logger.info("【支付回调】订单信息 : " + JSON.toJSONString(orderVO));
        if(orderVO.getOrderNo().startsWith(ServiceConstants.ORDER_TYPE_RESERVE_PREFIX)){// 预约订单
           logger.info("【支付回调】----------预约订单----------开始处理业务");
           /**
            *  1. 更新支付订单表，pay_bill
              2. 更新订单表的状态
              3. 更新分账表  分账金额、状态、实际支付金额
              4. 更新用户账变
              5. 订单履历
              */
           updateOrder(orderVO);

           logger.info("【支付回调】----------更新订单表成功----------");
//
//           updatePayBill(payBillVO);
//
//           logger.info("【支付回调】----------更新支付订单表成功----------");
//
//           updateDividend(orderVO);
//
//           logger.info("【支付回调】----------更新分账表成功----------");
//
//           generateChangeBalance(orderVO);
//
//           logger.info("【支付回调】----------生成用户账变成功----------");
//
//           updateChangeBalance(orderVO);
//
//           logger.info("【支付回调】----------更新用户账变表成功----------");
//
//           createOrderOper(orderVO);

           logger.info("【支付回调】----------创建订单履历成功----------");
        }else if (orderVO.getOrderNo().startsWith(ServiceConstants.ORDER_TYPE_REORDER_PREFIX)){// 续费订单
            logger.info("【支付回调】----------续费订单----------开始处理业务");

        } else if (orderVO.getOrderNo().startsWith(ServiceConstants.ORDER_TYPE_RECHARGE_PREFIX)) { // 充值订单
            logger.info("【支付回调】----------充值订单----------开始处理业务");

        }else {
            throw new ServiceException("非法的订单类型");
        }


//        Order updateOrder = new Order();
//        updateOrder.setOrderId(originalOrder.getOrderId());
//
//        Long payChannel = originalOrder.getPayChannel();
//        ChannelVO channelVO = rlChannelService.selectChannelByChannelId(payChannel);
//
//        logger.info("【微信支付回调】预订单支付");
//
//        Boolean execute = transactionTemplate.execute(e -> {
//            updateOrder.setPaid("1");
//            updateOrder.setPayTime(DateUtils.getNowDate());
//            updateOrder.setPayType(payType);
//            updateOrder.setStatus(ServiceConstants.ORDER_STATUS_IN_USE);
//            /** 如果是续单的订单，结束原订单*/
//            if(originalOrder.getType().equals(ServiceConstants.ORDER_TYPE_REORDER)){
//                closeOriginalOrder(originalOrder, updateOrder);
//            }
//            logger.info("=================【微信支付回调】开始更新订单信息=================={}",JSON.toJSON(updateOrder));
//            int updateEtOrder = orderService.updateRlOrder(updateOrder);
//            if(updateEtOrder==0){
//                logger.error("【微信支付回调】更新订单信息失败");
//                throw new ServiceException("【微信支付回调】更新订单信息失败");
//            }
//            /** 更新分成状态 */
//
//            // 批量修改账变记录
//            generateChangeBalance(rlDividendDetailVOS,originalOrder);

//
//            /** 记录订单履历 */
//            if(!orderOperService.recordOrderHistory(originalOrder.getOrderNo(),ServiceConstants.ORDER_OPERATION_PAY,
//                    originalOrder.getStatus(),updateOrder.getStatus(),BigDecimal.ZERO,originalOrder.getPayFee(),originalOrder.getUserId(),originalOrder.getPhone(),"订单支付：已支付 "+originalOrder.getPayFee()+"元")){
//                throw new ServiceException("【订单履历-支付】更新订单信息失败");
//            }
//            logger.info("=================6666666666666666666666666==================");
//            logger.info("=================【微信支付回调】全部结束！！！！！==================");
//            return Boolean.TRUE;
//        });
//        if(!execute)throw new ServiceException("微信支付回调失败");
    }

    private void updateOrder(OrderVO orderVO) {


    }


    /** 结算原订单 */
    private void closeOriginalOrder(Order order, Order updateOrder) {
        if(order.getType().equals(ServiceConstants.ORDER_TYPE_REORDER)){
            String originalOrderNo = order.getOriginalOrderNo();
            Order originalOrder = orderService.selectRlOrderByOrderNo(originalOrderNo);
            Order updateOriginalOrder = new Order();
            updateOriginalOrder.setOrderId(originalOrder.getOrderId());
//            updateOriginalOrder.setStatus(ServiceConstants.ORDER_STATUS_ORDER_END);
            int i = orderService.updateRlOrder(updateOriginalOrder);
            if(i==1){
                logger.error("【微信支付回调】更新原订单状态成功");
            }
            updateOrder.setStatus(ServiceConstants.ORDER_STATUS_IN_USE);
        }
    }


    private BigDecimal getCost(ChannelVO channel,BigDecimal payFee) {
        // 获取到渠道成本
        BigDecimal costRate = channel.getCostRate();
        BigDecimal bigDecimal = costRate.divide(new BigDecimal(100), 6, BigDecimal.ROUND_HALF_UP);
        BigDecimal cost = bigDecimal.multiply(payFee).setScale(2, BigDecimal.ROUND_HALF_UP);
        logger.info("【保存资金流水记录--订单支付】 成本==============bigDecimal=====================："+bigDecimal);
        logger.info("【保存资金流水记录--订单支付】 成本==============cost=====================："+cost);
        return cost;
    }



    private void asynchronousSaveCallbackLog(CallbackLog etCallbackLog) {
        //开异步线程保存回调参数
        scheduledExecutorService.schedule(() -> {
            logger.info("【微信支付回调】异步保存回调参数");
            etCallbackLog.setCreateTime(DateUtils.getNowDate());
            int i = callbackLogMapper.insertEtCallbackLog(etCallbackLog);
            if(i>0){
                logger.info("【微信支付回调】异步保存回调参数成功");
            }
        }, 0, TimeUnit.SECONDS);
    }


    /**
     * 微信退款回调
     */
    @Override
    @Transactional
    public void weChatRefund(HttpServletRequest request) {
        String body = HttpUtils.getBody(request);
        CallbackLog etCallbackLog = new CallbackLog();
        etCallbackLog.setBody2(body);
        etCallbackLog.setType("2");
        logger.info("【微信退款回调】接收对象 : " + JSON.toJSONString(body));
        // 解析通知数据
        Notification notification = JSON.parseObject(body, Notification.class);
        logger.info("【微信退款回调】转换成notification: " + JSON.toJSONString(notification));
        // 退款成功通知
        if (NotifyEventType.REFUND_SUCCESS.getValue().equals(notification.getEventType())) {
            // 验签、解密并转换成 RefundNotification
            RefundNotification refundNotification = checkAndParse(request, body, RefundNotification.class);
            logger.info("【微信退款回调】转换成RefundNotification: " + JSON.toJSONString(refundNotification));
            etCallbackLog.setBody(JSON.toJSONString(refundNotification));
            if (Status.SUCCESS.equals(refundNotification.getRefundStatus())) {
                // 退款成功后  改变 etRefundService
                // 如果outRefundNo没有值，说明是押金退款，直接新增退款记录，  还是在还车时新建退款记录？
                String outRefundNo = refundNotification.getOutRefundNo();
                Refund rlRefund = new Refund();
                rlRefund.setRefundNo(outRefundNo);
                rlRefund.setRefundResult(Constants.SUCCESS2);
                int i = etRefundService.updateEtRefundByRefundNo(rlRefund);
                if(i==0){
                    logger.error("【微信退款回调】更新退款单失败");
                    throw new ServiceException("【微信退款回调】更新退款单失败");
                }
                logger.info("【微信退款回调】全部结束！！！！！");
            }
        }
        //异步保存回调日志
        asynchronousSaveCallbackLog(etCallbackLog);
    }

    /**
     * 验签并解析
     * @param request 请求
     * @param body 请求体
     * @param clazz 返回值类型
     */
    private <T> T checkAndParse(HttpServletRequest request, String body, Class<T> clazz) {
        // 构造 RequestParam
        RequestParam requestParam = new RequestParam.Builder()
                .serialNumber(request.getHeader("Wechatpay-Serial"))
                .nonce(request.getHeader("Wechatpay-Nonce"))
                .signature(request.getHeader("Wechatpay-Signature"))
                .timestamp(request.getHeader("Wechatpay-Timestamp"))
                .body(body)
                .build();
        // 验签
        String wechatpaySerial = request.getHeader("Wechatpay-Serial");
        NotificationParser notificationParser = getNotificationParser(wechatpaySerial);
        return notificationParser.parse(requestParam, clazz);
    }


    public NotificationParser getNotificationParser(String wechatpaySerial) {
        // 根据证书序列号获取运营商
        ChannelVO channelVO = rlChannelService.selectChannelBySerialNumber(wechatpaySerial);
        if(ObjectUtil.isNull(channelVO)){
            throw new ServiceException("【微信支付回调】根据证书序列号获取支付渠道失败");
        }else{
            NotificationConfig config = new RSAAutoCertificateConfig.Builder()
                    .merchantId(channelVO.getMerchantId())
                    .privateKeyFromPath(channelVO.getPrivateKeyPath())
                    .merchantSerialNumber(channelVO.getMerchantSerialNumber())
                    .apiV3Key(channelVO.getApiV3Key())
                    .build();
            return new NotificationParser(config);
        }
    }
}
